#ifndef bqtBindingQbHH
#define bqtBindingQbHH

#include <string>
#include "symtables.hh"

/* Used by lexer */
void SetDefVarType(char begin, char end, char type);
void SetMetaDynamic();
void SetMetaStatic();

bool IsMetaDynamic();

/* Used by parser */
char  HasTypeSuffix(const std::string& varname);
char  CalcTypeSuffix(const std::string& varname);
std::string StripTypeSuffix(const std::string& varname);

long double DeclareAsLabelInCurrentScope(const std::string& varname);
unsigned TranslateLabelToDataPtr(const long double l);

struct VarContext
{
    // Pointer to the definition. 0 = not defined.
    ctxp info;
    
    // True if it's imported using SHARED
    bool is_global;
    
    VarContext(): info(),is_global(false) {}
    
    operator bool() const { return info; }
};

UtypeP FindUserType(const std::string& name);
bool DefineUserType(const std::string& name, UtypeP type);

// Creates a type template for the use by DIM and Implicit definitions.
// Uses the type suffix to initialize the "Type" field. Leaves other fields at default.
TypeP CreateTypeTemplate(const std::string& name);

exprp FindConstant(const std::string& name);
bool DefineConstant(const std::string& name, exprp p);

// Check if the identifier has been defined in the given class
VarContext FindScalarType(const std::string& name); // typesuffix optionally allowed
VarContext FindArrayType(const std::string& name); // typesuffix optionally allowed

TypeP ImplicitlyDefineScalar(const std::string& name,
                             bool is_static=false,
                             bool is_shared=false); // typesuffix optionally allowed
TypeP ImplicitlyDefineArray(const std::string& name,
                            unsigned num_dimensions,
                            unsigned starting_index,
                            unsigned ending_index,
                            bool is_static=false,
                            bool is_shared=false); // typesuffix optionally allowed

// The DIM statement.
TypeP ExplicitlyDefine(const std::string& name, TypeP type,
                       bool is_static, bool is_shared);

/* Note: Array definition may fail if a SUB/FUNCTION exists by the same name.
 *       Scalar definition may fail if a CONST exists by the same name.
 *       Scalar definition may fail if a SUB/FUNCTION exists by the same name.
 *
 *       CONST definition may fail if a scalar exists by the same name.
 *       CONST definition may fail if a SUB/FUNCTION exists by the same name.
 *
 * Array definition will always check the $DYNAMIC state, but never allow redefinition.
 *
 * Note:
 *  DIM a!
 *  a% = 5   ' This generates a Duplicate definition
 *
 */

#endif
